"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _structureBuilder = _interopRequireDefault(require("@sanity/desk-tool/structure-builder"));

var _schemaCreator = _interopRequireDefault(require("part:@sanity/base/schema-creator"));

var _schemaType = _interopRequireDefault(require("all:part:@sanity/base/schema-type"));

var _client = _interopRequireDefault(require("part:@sanity/base/client"));

var _blockContent = _interopRequireDefault(require("./parts/schemas/blockContent"));

var _link = _interopRequireDefault(require("./parts/schemas/link"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

function _createForOfIteratorHelper(o, allowArrayLike) { var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"]; if (!it) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = it.call(o); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it.return != null) it.return(); } finally { if (didErr) throw err; } } }; }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

var onlyhtmlToSanityTypes = {
  'text': 'string',
  'image': 'image',
  'html': 'blockContent',
  'link': 'link',
  'icon': 'iconPicker',
  // TODO add support in interneto-core
  'choice': 'boolean',
  'reference': 'reference'
};

class OnlyHtml {
  constructor(blocks) {
    this.blocks = blocks.filter(block => block.fields.length > 0);
  }

  createSchema() {
    var documents = {};

    var _iterator = _createForOfIteratorHelper(this.blocks),
        _step;

    try {
      for (_iterator.s(); !(_step = _iterator.n()).done;) {
        var block = _step.value;

        if (block.id in documents) {
          console.warn('duplicate block, is this handled correctly?');
        }

        documents[block.id] = this._convertBlockToSanityDocument(block);
      }
    } catch (err) {
      _iterator.e(err);
    } finally {
      _iterator.f();
    }

    console.log('done converting blocks to schema'); // use default schema types and add auto-generated site specific ones on top

    var types = _schemaType.default;
    var schemas = Object.values(documents);
    types = types.concat(schemas);
    types = types.concat([(0, _blockContent.default)(_schemaType.default.map(t => t.name)), _link.default]);
    return (0, _schemaCreator.default)({
      name: 'default',
      types: types
    });
  }

  createDeskStructure() {
    var singletonDocuments = this.blocks.filter(block => isSingleton(block.type)).map(block => block.id);

    var collectionItems = _structureBuilder.default.documentTypeListItems().filter(listItem => !singletonDocuments.includes(listItem.getId()));

    var singletonItems = singletonDocuments.map(id => {
      return _structureBuilder.default.listItem().title(idToTitle(id)).child(_structureBuilder.default.editor().schemaType(id).documentId(id));
    });
    singletonDocuments.map( /*#__PURE__*/function () {
      var _ref = _asyncToGenerator(function* (documentId) {
        // in our case id and type are the same from singleton documents
        var doc = {
          _id: documentId,
          _type: documentId
        };
        yield _client.default.createIfNotExists(doc);
      });

      return function (_x) {
        return _ref.apply(this, arguments);
      };
    }());
    return _structureBuilder.default.list().title('Sections').items([...singletonItems, _structureBuilder.default.divider(), ...collectionItems]);
  }

  _convertBlockToSanityDocument(block) {
    var sanityFields = [];

    var _iterator2 = _createForOfIteratorHelper(block.fields),
        _step2;

    try {
      for (_iterator2.s(); !(_step2 = _iterator2.n()).done;) {
        var field = _step2.value;
        var sanityField = {
          name: field.id,
          title: idToTitle(field.id),
          type: onlyhtmlToSanityTypes[field.type] || field.type
        };

        if (field.type === 'reference' && field.options.multiple) {
          sanityField.type = 'array';
          sanityField.of = [{
            type: 'reference',
            to: [{
              type: field.options.target
            }]
          }];
        } else if (field.type === 'reference') {
          sanityField.to = [{
            type: field.options.target
          }];
        }

        sanityFields.push(sanityField);
      }
    } catch (err) {
      _iterator2.e(err);
    } finally {
      _iterator2.f();
    }

    if (block.type === 'collection' && block.hasCollectionItems === true) {
      sanityFields.push({
        title: 'Slug',
        name: 'slug',
        type: 'slug',
        options: {
          source: 'title',
          // best effort to automate slug creation from title if exists
          slugify: input => input.toLowerCase().replace(/[\s\/]+/g, '-')
        }
      });
    }

    if (block.type === 'collection') {
      // Works nice with https://www.sanity.io/plugins/order-documents
      sanityFields.push({
        name: 'order',
        title: 'Order',
        type: 'number',
        hidden: true
      });
    }

    var sanityDocument = {
      name: block.id,
      title: idToTitle(block.id),
      type: 'document',
      fields: sanityFields
    };

    if (isSingleton(block.type)) {
      sanityDocument.__experimental_actions = ['update', 'publish']; // dont allow creating more than one
    }

    return sanityDocument;
  }

}

exports.default = OnlyHtml;

function isSingleton(blockType) {
  return blockType === 'page' || blockType === 'section';
}

function idToTitle(id) {
  id = id.replace('_', ' ');
  id = id.split(' ').map(val => val.charAt(0).toUpperCase() + val.substr(1)).join(' ');
  return id;
}
//# sourceMappingURL=onlyhtml.js.map